/*
 * AN00219_headset_output.xc
 *
 *  Created on: 18. okt. 2021
 *      Author: oyvindteig
 */

#include <platform.h>
#include <xs1.h>
#include <string.h>
// #include <xclib.h>

#include "mic_array.h"
#include "mic_array_board_support.h"
#include "debug_print.h"
#include <timer.h>    // delay_milliseconds(200), XS1_TIMER_HZ etc

#include "i2c.h"
#include "i2s.h"

#include "AN00219_headset_output.h"

#define MASTER_TO_PDM_CLOCK_DIVIDER 4
#define MASTER_CLOCK_FREQUENCY      24576000
#define PDM_CLOCK_FREQUENCY         (MASTER_CLOCK_FREQUENCY/(2*MASTER_TO_PDM_CLOCK_DIVIDER))
#define OUTPUT_SAMPLE_RATE          (PDM_CLOCK_FREQUENCY/(32*DECIMATION_FACTOR))

[[distributable]]
void i2s_handler (
        server  i2s_callback_if i2s, // From i2s_master0
        client  i2c_master_if   i2c,
        port    p_rst_shared,
        chanend c_audio)
{

    p_rst_shared <: 0xF; // Teig: scope: low for ages before this! With R71

    mabs_init_pll(i2c, ETH_MIC_ARRAY);
    // delay_milliseconds(10); // Teig. Did not help

    i2c_regop_res_t res;
    // The first thing I had done when I suspected the I2C was to print out all the I2Creturns
    // i2c_regop_res_tat the init section of i2s_handler. They were allI2C_REGOP_SUCCESS,
    // even with i2c_lib 4.0.0. But then, there is no parity or CRC checks here, and what
    // would still be open would be single bit errors in the data, even if the start and stop
    // conditions were successful.

    // INIT THE DAC AND PLL FOR THE HEADPHONES
    uint8_t data;

    // CS41L21_DAC_REG_01_CHIP_ID_AND_REVISION
    // Bit7     6        5        4        3        2       1       0
    // Chip_ID4 Chip_ID3 Chip_ID2 Chip_ID1 Chip_ID0 Rev_ID2 Rev_ID1 Rev_ID0
    data = i2c.read_reg (I2C_ADDRESS_OF_DAC, CS41L21_DAC_REG_01_CHIP_ID_AND_REVISION, res);
    debug_printf ("I2C res 1:%u", res);

    // CS41L21_DAC_REG_02_POWER_CONTROL_1 = POWER DOWN
    // Bit7     6        5        4        3        2        1        0
    // Reserved PDN_DACB PDN_DACA Reserved Reserved Reserved Reserved PDN
    data = i2c.read_reg (I2C_ADDRESS_OF_DAC, CS41L21_DAC_REG_02_POWER_CONTROL_1, res);
    debug_printf (" 2:%u", res);
    data |= 1; // Set Power Down (PDN) bit0 to 1=Enable
    res = i2c.write_reg (I2C_ADDRESS_OF_DAC, CS41L21_DAC_REG_02_POWER_CONTROL_1, data); // Power down
    debug_printf (" 3:%u", res);

    // CS41L21_DAC_REG_03_SPEED_CONTROL: Setting MCLKDIV2 high if using 24.576MHz.
    // Bit7 6      5      4       3        2        1        0
    // AUTO SPEED1 SPEED0 3-ST_SP Reserved Reserved Reserved MCLKDIV2
    data = i2c.read_reg (I2C_ADDRESS_OF_DAC, CS41L21_DAC_REG_03_SPEED_CONTROL, res);
    debug_printf (" 4:%u", res);
    data |= 1; // Set MCLK Divide By 2 (MCLKDIV2) bit0 to 1=Divide by 2
    res = i2c.write_reg (I2C_ADDRESS_OF_DAC, CS41L21_DAC_REG_03_SPEED_CONTROL, data);
    debug_printf (" 5:%u", res);

    // CS41L21_DAC_REG_10_MIXER_VOLUME_CONTROL_PCMA
    // Bit7     6       5        4        3        2        1        0
    // MUTE_    PCMMIXx PCMMIXx_ PCMMIXx_ PCMMIXx_ PCMMIXx_ PCMMIXx_ PCMMIXx_
    // PCMMIXx_ VOL6    VOL5     VOL4     VOL3     VOL2     VOL1     VOL0
    // 0        1       1        1        0        0        0        0
    data = 0b01110000; // 0x70 = 112. * 0.5 dB = 56 dB gain
    res = i2c.write_reg (I2C_ADDRESS_OF_DAC, CS41L21_DAC_REG_10_MIXER_VOLUME_CONTROL_PCMA, data);
    debug_printf (" 6:%u", res);

    // CS41L21_DAC_REG_02_POWER_CONTROL_1 POWER UP
    // Bit7     6        5        4        3        2        1        0
    // Reserved PDN_DACB PDN_DACA Reserved Reserved Reserved Reserved PDN
    data = i2c.read_reg (I2C_ADDRESS_OF_DAC, CS41L21_DAC_REG_02_POWER_CONTROL_1, res);
    debug_printf (" 7:%u", res);
    data &= ~1; // and_eq 0xFE is clear Power Down (PDN) bit0 0=Disable
    res = i2c.write_reg (I2C_ADDRESS_OF_DAC, CS41L21_DAC_REG_02_POWER_CONTROL_1, data); // Power up
    debug_printf (" 8:%u", res); // Power up

    // CS2100CP_PLL_REG_03_DEVICE_CONFIGURATION_1 INIT THE PLL.
    // IF FOR NOTHING ELSE; GOOD FOR ENSURING WORKING CHIP!
    // Bit7     6        5        4        3        2          1          0
    // RModSel2 RModSel1 RModSel0 Reserved Reserved AuxOutSrc1 AuxOutSrc0 EnDevCfg1
    // 0        0        0        0        0        0          0          0 is init value anyhow!
    //                                              0          0            Timing Reference Clock = RefClk
    res = i2c.write_reg (I2C_ADDRESS_OF_PLL, CS2100CP_PLL_REG_03_DEVICE_CONFIGURATION_1, 0); // RefClk to AUX_AOUT
    debug_printf (" 9:%u\n", res);

    while (1) {
        select { // Coming from i2s_master as "callbacks"

            case i2s.init (
                    i2s_config_t &?i2s_config,  // ? no if (!isnull(i2s_config))
                    tdm_config_t &?tdm_config): // ? no if (!isnull(tdm_config))
            {
                i2s_config.mode = I2S_MODE_LEFT_JUSTIFIED;
                i2s_config.mclk_bclk_ratio = (MASTER_CLOCK_FREQUENCY/OUTPUT_SAMPLE_RATE)/64;
            } break;

            case i2s.restart_check() -> i2s_restart_t restart: {
                restart = I2S_NO_RESTART;
            } break;

            case i2s.receive (size_t index, int32_t sample): {
                // No code
            } break;

            case i2s.send (size_t index) -> int32_t sample: {
                c_audio:> sample;
            } break;
        }
    }
}


